import { Canvas, CanvasView, createRectF } from '@nativescript-community/ui-canvas';
import { ImageSource, Screen, Utils } from '@nativescript/core';
import { SUPPORTED_CANVAS_IMAGE_FORMATS } from '../helpers';
const styleProxyHandler = {
    get(target, name, receiver) {
        const view = target.view;
        if (view != null) {
            if (name === 'width') {
                return Utils.layout.toDeviceIndependentPixels(view.getMeasuredWidth());
            }
            if (name === 'height') {
                return Utils.layout.toDeviceIndependentPixels(view.getMeasuredWidth());
            }
        }
        return Reflect.get(target, name, receiver);
    },
    set(target, name, value) {
        if (name === 'width' || name === 'height') {
            target[name] = typeof value === 'string' && value.endsWith('px') ? parseFloat(value) : value;
        }
        else {
            target[name] = value;
        }
        return true;
    },
};
class HTMLCanvasViewBase extends CanvasView {
    constructor() {
        super();
        this._mStyle = new Proxy(super.style, styleProxyHandler);
    }
    getContext(contextId, contextAttributes) {
        if (this._isControlledByOffscreen) {
            throw new Error(`Failed to execute 'getContext' on 'HTMLCanvasView': Cannot get context from a canvas that has transferred its control to offscreen.`);
        }
        const self = this;
        let cl;
        switch (contextId) {
            case '2d':
                cl = CanvasRenderingContext2D;
                break;
            case 'bitmaprenderer':
                cl = ImageBitmapRenderingContext;
                break;
            default:
                cl = null;
                break;
        }
        if (cl == null) {
            return null;
        }
        if (this._context2D != null && this._context2D instanceof cl) {
            return this._context2D;
        }
        const context = new cl();
        Object.defineProperty(context, 'canvas', {
            get() {
                return self;
            },
        });
        this._context2D = context;
        return this._context2D;
    }
    enableOffscreenBuffer() {
        if (this._offscreenContext == null) {
            const scale = Screen.mainScreen.scale;
            this._offscreenContext = new Canvas(this.width * scale, this.height * scale);
            this._offscreenContext.scale(scale, scale);
        }
    }
    disableOffscreenBuffer() {
        if (this._offscreenContext != null) {
            this._offscreenContext.release();
            this._offscreenContext = null;
        }
    }
    isOffscreenBufferEnabled() {
        return !!this._offscreenContext;
    }
    drawOffscreenBuffer() {
        if (this._offscreenContext == null) {
            console.warn('This element does not have an offscreen buffer to draw');
            return;
        }
        const nativeContext = this._nativeContextRef.deref();
        const rect = createRectF(0, 0, this.width, this.height);
        nativeContext.drawBitmap(this._offscreenContext.getImage(), null, rect, null);
    }
    transferControlToOffscreen() {
        if (this._context != null) {
            throw new Error(`Failed to execute 'transferControlToOffscreen' on 'HTMLCanvasElement': Cannot transfer control from a canvas that has a rendering context.`);
        }
        if (this._isControlledByOffscreen) {
            throw new Error(`Failed to execute 'transferControlToOffscreen' on 'HTMLCanvasElement': Cannot transfer control from a canvas for more than one time.`);
        }
        this._isControlledByOffscreen = true;
        return new OffscreenCanvas(this.width, this.height);
    }
    toDataURL(type, encoderOptions) {
        if (this._offscreenContext == null) {
            console.warn(`'Method 'toDataURL' can only be used with offscreen buffer enabled. Please use 'enableOffscreenBuffer' and 'drawOffscreenBuffer' methods`);
            return null;
        }
        const prefix = 'data:';
        if (this.width === 0 || this.height === 0) {
            return prefix;
        }
        if (type == null || !SUPPORTED_CANVAS_IMAGE_FORMATS.includes(type)) {
            type = 'image/png';
        }
        const format = type != null ? type.split('/')[1] : 'png';
        const quality = encoderOptions ? encoderOptions * 100 : 100;
        const imgSource = new ImageSource(this._offscreenContext.getImage());
        const base64String = imgSource.toBase64String(format, quality);
        return `${prefix + type};base64,${base64String}`;
    }
    get nativeContext() {
        return this._lastCanvas;
    }
    onDraw(canvas) {
        this._lastCanvas = canvas;
        super.onDraw(canvas);
    }
    get style() {
        return this._mStyle ?? super.style;
    }
    set style(val) {
        this._mStyle = new Proxy(val, styleProxyHandler);
    }
}
export { HTMLCanvasViewBase };
//# sourceMappingURL=html-canvas-view-common.js.map